# == Define: puppet_enterprise::amq::config::authorization_plugin_entry
#
# Custom define for creating and managing an `authorization entry` for the
# `authorizationPlugin`.
#
# In ActiveMQ we use a number of operations which you can associate with user
# roles and either individual queues or topics or you can use wildcards to
# attach to hierarchies of topics and queues.
#
# === Parameters
#
# [*admin*]
#   You can lazily create the destination if it does not yet exist.
#   This allows you fine grained control over which new destinations can be
#   dynamically created in what part of the queue/topic hierarchy.
#
# [*destination_type*]
#   One of the following: topic, queue, tempQueue or tempTopic
#
# [*read*]
#  This user or group can browse and consume from the destination.
#
# [*target_destination*]
#   The topic or queues destination. can either be individual queues or topics,
#   or you can use wildcards.
#
# [*write*]
#   This user or group can send messages to the destination.
#
# [*brokername*]
#   The name of the ActiveMQ broker whose configuration this entry belongs to.
#   Defaults to the agents client certificate name.
#
# [*entry_ensure*]
#   Whether to ensure that this entry is present in the `authorizationPlugin` or abset.
#   Valid values are one of: preset, absent, true, false.
#
define puppet_enterprise::amq::config::authorization_plugin_entry(
  $admin,
  $destination_type,
  $read,
  $target_destination,
  $write,
  $brokername   = $::clientcert,
  $entry_ensure = 'present',
){
  pe_validate_re($destination_type, '^(topic|queue|tempTopic|tempQueue)$')
  pe_validate_re($entry_ensure, '^(present|absent|true|false)$')

  $broker_context = "/files/etc/puppetlabs/activemq/activemq.xml/beans/broker[#attribute/brokerName='${brokername}']"
  $plugin_context = "${broker_context}/plugins/authorizationPlugin"
  $entry_context  = "${plugin_context}/map/authorizationMap/authorizationEntries/authorizationEntry[#attribute/${destination_type}='${target_destination}']"

  if $entry_ensure =~ /^(absent|false)$/ {
    $changes = "rm ${entry_context}"
  } else {
    $_target_destination = "set #attribute/${destination_type} '${target_destination}'"
    $_admin              = "set #attribute/admin '${admin}'"
    $_read               = "set #attribute/read '${read}'"
    $_write              = "set #attribute/write '${write}'"

    $changes = pe_delete_undef_values(pe_flatten([
      $_target_destination,
      $_admin,
      $_read,
      $_write,
    ]))
  }

  augeas { "${brokername}: AMQ authorizationPlugin entry: ${title}":
    incl    => '/etc/puppetlabs/activemq/activemq.xml',
    lens    => 'ActiveMQ_XML.lns',
    context => $entry_context,
    changes => $changes,
    require => Puppet_enterprise::Amq::Config::Broker[$brokername],
    notify  => Service['pe-activemq'],
  }
}
