# == Define: Puppet_enterprise::mcollective::client
#
# Defined type for creating and managing an mcollective client.
# An mcollective client consists of several parts:
#   - The mcollective-client package
#   - The system user
#   - The certificates for communicating with amq and singing messages
#   - The mcollective client config file
#
# If you would like to manage the system user by yourself, you can pass in
# `create_user => false`. If you do this, make sure that you pass in a valid `home_dir`.
#
# === Parameters
#
# [*log_file*]
#   Path to where the mcollective client should log to.
#
# [*cert_name*]
#   The name to use for the clients certificate. This should be a unique name across your infrastructure.
#   Defaults to `$title`.
#
# [*client_name*]
#   The name of the mcollective client. If you have left `manage_user => true`, than it will also be the name
#   of the system user created. Defaults to `$title`.
#
# [*home_dir*]
#   The home directory of the mcollective client user. This is where all the configuration and certs will live.
#
# [*create_user*]
#   Boolean. Whether or not to create and manage the system user. If false, ensure you pass in a valid `home_dir`.
#
# [*stomp_password*]
#   Password for the activemq connecter to use.
#
# [*stomp_port*]
#   Port for the activemq connector to use.
#
# [*stomp_user*]
#   The username for the activemq connector to use.
#
# === Examples
#
#   For examples, look at
#   `puppet_enterprise::profiles::mcollective::peadmin`
#   `puppet_enterprise::profiles::mcollective::console`
#
define puppet_enterprise::mcollective::client(
  $activemq_brokers,
  $logfile,
  $cert_name              = $title,
  $client_name            = $title,
  $keypair_name           = $title,
  $create_user            = $puppet_enterprise::params::mco_create_client_user,
  $home_dir               = "/var/lib/${title}",
  $stomp_password         = $puppet_enterprise::params::stomp_password,
  $stomp_port             = $puppet_enterprise::mcollective_middleware_port,
  $stomp_user             = $puppet_enterprise::params::stomp_user,
){
  include puppet_enterprise::packages
  include puppet_enterprise::params

  $mco_server_name = $puppet_enterprise::params::mco_server_name

  Package <| tag == 'pe-mco-client-packages' |>

  if $create_user {
    puppet_enterprise::mcollective::client::user { $client_name:
      home_dir => $home_dir,
    }
  }

  $cert_dir = "${home_dir}/.mcollective.d"

  puppet_enterprise::mcollective::client::certs { $cert_name:
    client_name     => $client_name,
    keypair_name    => $keypair_name,
    destination_dir => $cert_dir,
  }

  File {
    owner => $client_name,
    group => $client_name,
    mode  => '0600'
  }

  file { $logfile:
    ensure => file
  }

  # Template uses:
  # - $activemq_brokers
  # - $cert_dir
  # - $cert_name
  # - $logfile
  # - $stomp_password
  # - $stomp_port
  # - $stomp_user
  # - $mco_server_name
  file { "${home_dir}/.mcollective":
    content => template('puppet_enterprise/mcollective/client.cfg.erb'),
  }

  # Previous versions of PE used to lay down client.cfg in /etc/puppetlabs/mcollective/client.cfg
  # Remove it since its now placed in the clients homedir. Since this is a defined type, it could be used
  # multiple times on one node (for example an all in one master will have the profile console and peadmin).
  # to prevent duplicate declarations, check if this hasn't already been defined.
  if ! defined(File['/etc/puppetlabs/mcollective/client.cfg']) {
    file { '/etc/puppetlabs/mcollective/client.cfg':
      ensure => absent,
    }
  }
}
